From b94ba0d4e07635a0abddf0cba7548b10b47dad92 Mon Sep 17 00:00:00 2001
From: kobarity <kobarity@gmail.com>
Date: Fri, 30 Aug 2024 00:13:27 +0900
Subject: [PATCH] Fix Python prompt detection when 'exec-path' is buffer local

* lisp/progmodes/python.el (python-with-temp-buffer): New macro
same as 'with-temp-buffer', except the variable 'exec-path' is
kept in the new buffer.
(python-shell-prompt-detect): Use it. (Bug#72849)

Co-authored-by: Evgenii Klimov <eugene.dev@lipklim.org>
---
 lisp/progmodes/python.el | 12 +++++++++++-
 1 file changed, 11 insertions(+), 1 deletion(-)

diff --git a/lisp/progmodes/python.el b/lisp/progmodes/python.el
index a00289d6de9..480713c3aca 100644
--- a/lisp/progmodes/python.el
+++ b/lisp/progmodes/python.el
@@ -3088,6 +3088,16 @@ 'python--parse-json-array
         (json-read-from-string string))))
   "Parse the JSON array in STRING into a Lisp list.")
 
+(defmacro python-with-temp-buffer (&rest body)
+  "Python mode specific version of `with-temp-buffer'.
+Call `with-temp-buffer' with BODY.  The variable `exec-path' is kept in
+the new buffer even if it is buffer local."
+  (declare (indent 0) (debug t))
+  `(let ((python--exec-path exec-path))
+     (with-temp-buffer
+       (let ((exec-path python--exec-path))
+         ,@body))))
+
 (defun python-shell-prompt-detect ()
   "Detect prompts for the current `python-shell-interpreter'.
 When prompts can be retrieved successfully from the
@@ -3116,7 +3126,7 @@ python-shell-prompt-detect
              (interpreter python-shell-interpreter)
              (interpreter-arg python-shell-interpreter-interactive-arg)
              (output
-              (with-temp-buffer
+              (python-with-temp-buffer
                 ;; TODO: improve error handling by using
                 ;; `condition-case' and displaying the error message to
                 ;; the user in the no-prompts warning.
-- 
2.34.1

